#!/usr/bin/perl
use strict;
use warnings;
use lib './lib';
use Benchmark;
use Dumpvalue;
use encoding 'utf8';
use XML::Simple;
use XML::Parser;

die "Usage: ./bench.pl xmlfile\n" if(!@ARGV);

my $xml;
{
	open(IN,$ARGV[0]) or die;
	local $/ = undef;
	$xml = <IN>;
	close(IN);
}

sub simpleFast {
	$XML::Simple::PREFERRED_PARSER = 'XML::Parser';
	my $p = XML::Simple->new();
	my $r = $p->XMLin($xml,ForceArray=>1,KeyAttr=>{}) or die;
}

sub simpleSlow {
	#$XML::Simple::PREFERRED_PARSER = undef;
	$XML::Simple::PREFERRED_PARSER = 'XML::SAX::PurePer';
	my $p = XML::Simple->new();
	my $r = $p->XMLin($xml,ForceArray=>1,KeyAttr=>{}) or die;
}

sub xmlParser {
	my $p = new XML::Parser(Style=>'Simple');
	my $r = $p->parse($xml);
}

#my $ret;
#$ret = simpleFast();
#$ret = simpleSlow();
#$ret = xmlParser();
#Dumpvalue->new->dumpValue($ret);
#die;

timethese(500,{
	SimpleFast  => 'simpleFast($ARGV[0])',
	SimpleSlow  => 'simpleSlow($ARGV[0])',
	XmlParser   => 'xmlParser($ARGV[0])',
});


=bench result 

$ ./bench.pl sampleSmall.xml
Benchmark: timing 500 iterations of SimpleSlow, SimpleFast, XmlParser...
  SimpleSlow: 32 wallclock secs ( 1.66 usr + 30.45 sys = 32.11 CPU)
  SimpleFast:  2 wallclock secs ( 0.82 usr +  0.76 sys =  1.58 CPU)
   XmlParser:  1 wallclock secs ( 0.93 usr +  0.16 sys =  1.09 CPU)


$ ./bench.pl sampleLarge.xml
Benchmark: timing 500 iterations of SimpleSlow, SimpleFast, XmlParser...
  SimpleSlow: 1068 wallclock secs (202.55 usr + 865.64 sys = 1068.19 CPU)
  SimpleFast:   40 wallclock secs ( 39.47 usr +   0.47 sys =   39.94 CPU)
   XmlParser:   20 wallclock secs (  0.36 usr +  20.39 sys =   20.75 CPU)

=cut